/*

Deklaracja podstawowego interfejsu, czcego ogln ide dziaania protokou Jabber z programem
Konnekt. Powinna by dobrym punktem startowym do czenia dowolnej biblioteki obsugujcej
protok Jabbero-podobny z Konnektem.
Kod zosta wzity z maymi modyfikacjami z wtyczki kJabber.

Ustawienia przechowywane s w tablicy z identyfikatorami tekstowymi (name) w formacie:
this->dtPrefix + "/" + nazwa_zmiennej
Standardowo ustawiane i uywane s tylko zmienne user , pass i status

Klasa musi by zbudowana tak, eby bya kompatybilna przynajmniej z dwiema aktualnie wykorzystujcymi
j wtyczkami - kJabber i DwuTlenek. Dodatkowo bez wikszych zmian , powinno da
si uy dwch rnych pocze rwnolegle (Gdy Konnekt bdzie to ju supportowa).


	* License information can be found in license.txt.
    * Please redistribute only as a whole package.


(C) 2004 Rafa Lindemann - www.STAMINA.eu.org - www.KONNEKT.info

*/

#pragma once
#ifndef __JABBER_CLASS__H__
#define __JABBER_CLASS__H__

namespace kJabber {
	enum enSynchDirection {
		sd_disabled = 0 ,
		sd_toServer = 1 ,
		sd_toClient = 2
	};
	enum enConnectionState {
		cs_offline , 
		cs_connecting , 
		cs_online ,
		cs_disconnecting
	};
	const int IMsetTimer = 12010; // id komunikatu, po ktrym powinien zosta ustawiony timer od pingowania...
	const int pingInterval = 30000; // ping co p minuty (max. opnienie - pingInterval)
	class cJabberBase {
	public:
		cJabberBase(cCtrl * ctrl); // inicjalizacja
		virtual ~cJabberBase();

		/* Podstawowe funkcje obsugujce "logik" protokou Jabber 
		   Teoretycznie, dla wszystkich jabbero-podobnych protokow nie trzeba
		   modyfikowa ich dziaania (z wyjtkiem StartSSL, ktrego nie ma tu z czystego lenistwa ;))
		
		*/

		enConnectionState GetState() {return state;}
		char GetSlotID() {return slotID;}
		unsigned int GetSlot() {return slotID - '0';}
		unsigned int GetNet() {return net;}
		cCtrl * GetPlug() {return plug;}

		virtual void Lock();
		virtual void Unlock();

		/** Wrapper dla CreateThread */
		static unsigned int __stdcall ListenThread_(cJabberBase * jab) {
			return jab->ListenThread();
		}
		/** Nawizywanie poczenia z sieci */
		virtual void Connect();
		/** Wczenie obsugi SSL */
		virtual void StartSSL();
		/** Wczenie obsugi Proxy */
		virtual bool StartProxy();
		/** Rozczenie */
		virtual void Disconnect();
		/** Stan poczenia */
		virtual bool Connected() = 0;
		/** Czy lista jest ju zsynchronizowana? */
		virtual bool IsSynchronized() {
			return synchronized;
		}
		/** Ustawia status dla sesji */
		virtual void SetStatus(int status , bool set , const char * info = 0);
		/** Wyszukiwanie kontaktu z podanym JIDem */
		virtual int FindContact(const CStdString & jid);

		void IgnoreMessage(cMessage * m , int cnt);

		/* Obsuga komunikatw Konnekta 
		   Przynajmniej czciowo powinno si pokrywa...
		*/

		/** Obsuga komunikatw API konnekta */
		virtual int IMessageProc(sIMessage_base * msgBase);
		/** Obsuga komunikatw dla akcji konfiguracji */
		virtual int ActionCfgProc(sUIActionNotify_base * anBase) {return 0;}
		/** Obsuga komunikatw dla pozostaych akcji */
		virtual int ActionProc(sUIActionNotify_base * anBase);

		/** Przygotowanie interfejsu */
		virtual void UIPrepare();
		/** Rejestrowanie kolumn */
		virtual void SetColumns();
		/** Inicjacja wyszukiwania kontaktu */
		virtual void CntSearch(sCNTSEARCH * sr);
		/** Inicjacja pobierania informacji o kontakcie */
		virtual bool VCardRequest(int cntID);
		/** Inicjacja zapisywania informacji o uytkowniku */
		virtual bool VCardSet();

	/* W odpowiedzi na okrelone zdarzenia, trzeba wywoa odpowiednie funkcje
	   zdarze...

	   Klasa protokou musi w peni sama zaj si:
	    - przyjmowaniem wiadomoci i wstawianiem ich do kolejki...
		- obsug zmian stanw kontaktw i informoaniem o autoryzacji
		- obsug znalezionych w katalogu kontaktw
	*/

		void OnConnected();
		void OnRosterAnswer();
		void OnRosterItemRemoving(const CStdString & jid);
		void OnRosterItemUpdated(const CStdString & jid , void * item);
		void OnRosterItemAdded(const CStdString & jid , void * item);
		/** W momencie otrzymania proby o autoryzacj (request==true), lub jej odmwienia */
		void OnPresenceRequest(const CStdString & jid , bool request);
		
		void OnSearchAnswer();
		void OnVCardRequestAnswer();
		void OnVCardSetAnswer();


	/* Kada jabberowa klasa musi tutaj okreli swoje zachowanie */

	/* Interfejs niszego poziomu */

	/* Pobieranie zmiennych i ustawie */
		virtual const CStdString GetHost() = 0;
		virtual int GetPort() = 0;
		virtual const CStdString GetUID() = 0;
		virtual void GetLoginAndPass(CStdString & login , CStdString & pass);
		virtual enSynchDirection CFGsynchDirection() = 0;
		virtual int CFGsynchAddContacts() = 0;
		virtual int CFGsynchRemoveContacts() = 0;
		virtual int CFGautoSubscribe() = 0;
		virtual int CFGautoAuthorize() = 0;
		virtual SOCKET GetSocket() {return 0;}
		virtual const CStdString & GetName() {static CStdString name="Jabber"; return name;}

	/* Wykonywanie konkretnych operacji */

		/** Gwny wtek do nawizywania/obsugi poczenia */
		virtual unsigned int ListenThread(void) = 0;
		/** Wysya nowy status wasny... */
		virtual bool SendPresence(const CStdString & to , int status , const char * info) = 0;
		virtual bool SendSubRequest(const CStdString & to , bool subscribe) = 0;
		virtual bool SendSubscribed(const CStdString & to , bool subscribed) = 0;
		virtual bool SendCntSearch(sCNTSEARCH * sr) = 0;
		virtual bool SendVCardRequest(int cntID) = 0;
		virtual bool SendVCardSet(bool window) = 0;
		virtual bool SendMessage(cMessage * m) = 0;
		virtual bool RosterDeleteJID(const CStdString & jid) = 0;
		virtual bool RosterContainsJID(const CStdString & jid) = 0;

		static void CALLBACK PingTimerRoutine(cJabberBase * jab, DWORD l,  DWORD h);
		virtual bool SendPing() = 0;

		/** Rozcza poczenie sieciowe */
		virtual void DisconnectSocket() = 0;

		/** Aktualizuje kontakt w rosterze */
		virtual void SetRosterItem(int cntID) = 0;
		/** Aktualizuje kontakt z rostera. Wskanik do item przekazywany jest ze zdarze */
		virtual void SetContact(int cntID , void * item) = 0;
		/** Ustala (w razie potrzeby) specjaln ikon statusu kontaktu */
		virtual void SetCntIcon(int cntID , int status , const CStdString & jid) = 0;


	protected:
		cCtrl * plug;
		char slotID; ///< Identyfikator "slotu". W tej chwili moe istnie i tak tylko jeden slot...
		int net; ///< Warto net wtyczki...
		CStdString dtPrefix; ///< Prefix nazw kolumn w bazie
		HANDLE thread;
		int currentStatus;
		CStdString currentStatusInfo;
		bool inAutoAway;
		bool synchronized;
		// aktualnie usuwany kontakt (ID). -1 - aden kontakt nie jest usuwany
		int removingCnt;
		CRITICAL_SECTION cs;
		sDIALOG_long * dlgSearch , * dlgVCardReq , * dlgVCardSet;
		enConnectionState state;
		HANDLE timerPing;
		__time64_t lastPing;

	/* identyfikatory kontrolek */

	};

	class cExceptionQuiet {
	private:
		const cJabberBase * jab;
		CStdString where;
		CStdString msg;
	public:
		cExceptionQuiet(const cJabberBase * jab , const char * where = "" , const char * msg = "") {
			this->msg = msg;
			this->jab = jab;
			this->where = where;
		}
		const CStdString GetMsg() {return msg;}
		const CStdString GetWhere() {return where;}
		const cJabberBase * GetJab() {return jab;}
	};
	class cException: public cExceptionQuiet {
	public:
		cException(const cJabberBase * jab , const char * where , const char * msg):cExceptionQuiet(jab , where , msg) {
		}
	};

};

#endif